import os
import re

import requests
from prettytable import PrettyTable

import lib.utils as utils


class Entity:
    def __init__(self, options, auth):
        self.options = options
        self.auth = auth
        self.is_json = self.options.display == "JSON"

    def list(self):
        r = requests.get(os.path.join(self.options.url, "entity/list"),
                         auth=self.auth, params={"detail": True, "all": self.options.all})
        if r.status_code != 200:
            raise RuntimeError('incorrect result from \'entity/list\' service: %s' % r.text)
        result = r.json()
        if not self.is_json:
            row = ["ID", "Code", "Name", "Email", "Description"]
            if self.options.all:
                row.append("Active")
            res = PrettyTable(row)
            for r in sorted(result, key=lambda r: r["id"]):
                r_row = [r["id"], r["entity_code"], r["name"], r["email"], r["description"]]
                if self.options.all:
                    r_row.append(r["active"])
                res.add_row(r_row)
        else:
            res = result
        return res

    def send_data(self, data):
        r = requests.post(os.path.join(self.options.url, "entity/" + self.options.subcmd),
                          auth=self.auth, json=data)
        if r.status_code != 200:
            raise RuntimeError(
                'incorrect result from \'entity/%s\' service: %s' % (self.options.subcmd, r.text))
        return r

    def add(self):
        data = {
            'name': self.options.name,
            'email': self.options.email,
            'tel': self.options.tel,
            'address': self.options.address,
            'description': self.options.description
        }

        if not re.match(r"^[A-Z]{2}$", self.options.entity_code):
            raise ValueError('entity_code should be [A-Z]{2}')
        data['entity_code'] = self.options.entity_code

        r = self.send_data(data)
        res = "ok new entity: name=%s -> id=%d" % (self.options.name, r.json())
        return res

    def modify(self):
        data = {
            'name': self.options.name,
            'email': self.options.email,
            'tel': self.options.tel,
            'address': self.options.address,
            'description': self.options.description
        }

        status, entity_id = utils.get_entity(self.options.url, self.auth, self.options.entity)
        if not status:
            raise RuntimeError(entity_id)
        data['entity_id'] = entity_id

        r = self.send_data(data)
        res = 'ok'
        return res

    def disable(self):
        data = {}

        status, entity_id = utils.get_entity(self.options.url, self.auth, self.options.entity)
        if not status:
            raise RuntimeError(entity_id)
        data['entity_id'] = entity_id

        r = self.send_data(data)
        res = 'ok'
        return res

    def enable(self):
        data = {}

        status, entity_id = utils.get_entity(self.options.url, self.auth, self.options.entity)
        if not status:
            raise RuntimeError(entity_id)
        data['entity_id'] = entity_id

        r = self.send_data(data)
        res = 'ok'
        return res

    def execute_command(self):
        result = None
        if self.options.subcmd == "list":
            result = self.list()
        if self.options.subcmd == "add":
            result = self.add()
        if self.options.subcmd == "modify":
            result = self.modify()
        if self.options.subcmd == "disable":
            result = self.disable()
        if self.options.subcmd == "enable":
            result = self.enable()
        return result



import os

import requests
from prettytable import PrettyTable

import lib.utils as utils


class Group:
    def __init__(self, options, auth):
        self.options = options
        self.auth = auth
        self.is_json = self.options.display == "JSON"

    def list(self):
        status_code, result = utils.group_list(self.options.url, self.auth)
        if status_code != 200:
            raise RuntimeError(result)
        if not self.is_json:
            res = PrettyTable(["ID", "Name", "Roles", "Users"])
            res.align["Name"] = "l"
            res.align["Roles"] = "l"
            res.align["Users"] = "l"
            for g in result:
                name = g["entity_code"] + ":" + g["name"]
                roles = []
                for r in g["roles"]:
                    if "entity_code" in r:
                        roles.append(r["entity_code"] + ":" + r["name"])
                    else:
                        roles.append(r["name"])
                users = []
                for u in g["users"]:
                    u["name"] = u["first_name"] + " " + u["last_name"]
                    users.append(u["tid"] + "(" + u["name"] + ")")
                res.add_row([g["id"], name, " ".join(roles), " ".join(users)])
        else:
            res = result
        return res

    def send_data(self, data):
        r = requests.post(os.path.join(self.options.url, "group/" + self.options.subcmd),
                          auth=self.auth, json=data)
        if r.status_code != 200:
            raise RuntimeError(
                'incorrect result from \'group/%s\' service: %s' % (self.options.subcmd, r.text))
        res = 'ok'
        return res

    def get_data(self):
        data = {
            'name': self.options.name
        }
        if self.options.users is not None:
            users_id = []
            for user in self.options.users:
                status, user_id = utils.get_user(self.options.url, self.auth, user)
                if not status:
                    raise RuntimeError(user_id)
                users_id.append(user_id)
            data['users'] = users_id
        if self.options.roles is not None:
            roles_id = []
            for role in self.options.roles:
                status, role_id = utils.get_role(self.options.url, self.auth, role)
                if not status:
                    raise RuntimeError(role_id)
                roles_id.append(role_id)
            data['roles'] = roles_id
        return data

    def add(self):
        data = self.get_data()
        if self.options.entity is not None:
            status, entity_id = utils.get_entity(self.options.url, self.auth, self.options.entity)
            if not status:
                raise RuntimeError(entity_id)
            data['entity_id'] = entity_id

        res = self.send_data(data)

        return res

    def modify(self):
        data = self.get_data()
        status, group_id = utils.get_group(self.options.url, self.auth, self.options.group)
        if not status:
            raise RuntimeError(group_id)
        data['group_id'] = group_id

        res = self.send_data(data)

        return res

    def delete(self):
        data = {}
        status, group_id = utils.get_group(self.options.url, self.auth, self.options.group)
        if not status:
            raise RuntimeError(group_id)
        data['group_id'] = group_id

        res = self.send_data(data)

        return res

    def execute_command(self):
        result = None
        if self.options.subcmd == "list":
            result = self.list()
        if self.options.subcmd == "add":
            result = self.add()
        if self.options.subcmd == "modify":
            result = self.modify()
        if self.options.subcmd == "delete":
            result = self.delete()
        return result



import os
import sys
from datetime import datetime

import requests
import six
from prettytable import PrettyTable

from lib.utils import write_to_stdout


class Resource:
    def __init__(self, options, auth):
        self.options = options
        self.auth = auth
        self.is_json = self.options.display == "JSON"

    def list(self):
        path = self.options.path
        if path and len(path.split(':')) != 2:
            raise ValueError('invalid path format: %s (should be storage:path)' % path)
        data = {'path': path, 'service': self.options.service}
        if "entity" in self.options and self.options.entity:
            data["entity"] = self.options.entity
        r = requests.get(os.path.join(self.options.url, "resource/list"), auth=self.auth, data=data)
        if r.status_code != 200:
            raise RuntimeError('incorrect result from \'resource/list\' service: %s' % r.text)
        result = r.json()
        if not self.is_json:
            if self.options.path is None or self.options.path == '':
                res = PrettyTable(['Pool', 'Entity', 'Name', 'Type', 'Description'])
                res.align["Name"] = "l"
                res.align["Type"] = "l"
                res.align["Pool"] = "l"
                res.align["Description"] = "l"
                for r in result:
                    entity = r["entity"] if r["entity"] != "CONF_DEFAULT" else ""
                    res.add_row([r["pool"], entity, r["name"] + ":", r["type"], r["description"]])
            elif self.options.aggr:
                res = PrettyTable(['Type', 'Path', 'Suffixes'])
                res.align["Path"] = "l"
                res.align["Suffixes"] = "l"
                files = {}
                if not isinstance(result, list):
                    result = [result]
                for k in result:
                    if type(k) == dict:
                        k = k['key']
                    if k.endswith('/'):
                        res.add_row(['dir', k, ''])
                    else:
                        suffix = ""
                        if k.endswith(".gz"):
                            suffix = ".gz"
                            k = k[:-3]
                        p = k.rfind(".")
                        if p != -1:
                            suffix = k[p:] + suffix
                            k = k[:p]
                        if k not in files:
                            files[k] = []
                        files[k].append(suffix)
                for k, v in six.iteritems(files):
                    res.add_row(['file', k, ', '.join(sorted(v))])
            else:
                res = PrettyTable(['Type', 'Path', 'LastModified', 'Size (in byte)'])
                res.align["Path"] = "l"
                res.align["LastModified"] = "l"
                res.align["Size"] = "l"
                files = {}
                if not isinstance(result, list):
                    result = [result]
                for k in result:
                    meta = {}
                    if type(k) == dict:
                        meta = k
                        k = meta['key']
                    if k.endswith('/'):
                        res.add_row(['dir', k, '', ''])
                    else:
                        date = ''
                        if 'last_modified' in meta:
                            date = datetime.fromtimestamp(meta['last_modified']).strftime(
                                "%m/%d/%Y, %H:%M:%S")
                        size = ''
                        if 'size' in meta:
                            size = meta['size']
                        res.add_row(['file', k, date, size])
        else:
            res = result
        return res

    def get(self):
        params = {'path': self.options.path}
        if self.options.service is not None:
            params['service'] = self.options.service
        r = requests.get(os.path.join(self.options.url, "resource/file"),
                         auth=self.auth, params=params)
        if r.status_code != 200:
            raise RuntimeError('incorrect result from \'resource/file\' service: %s' % r.text)
        for chunk in r.iter_content(chunk_size=512 * 1024):
            if chunk:
                write_to_stdout(chunk)
        sys.exit(0)

    def execute_command(self):
        result = None
        if self.options.subcmd == "list":
            result = self.list()
        if self.options.subcmd == "get":
            result = self.get()
        return result



import logging
import os

import requests

import client.launcher as launcher
from lib import utils

launcher.LOGGER = logging.getLogger()


class Share:
    def __init__(self, options, auth):
        self.options = options
        self.auth = auth
        self.is_json = self.options.display == "JSON"

    def add(self):
        data = {"roles": []}
        for role in self.options.roles:
            status, role_id = utils.get_role(self.options.url, self.auth, role)
            if not status:
                raise RuntimeError(role_id)
            data['roles'].append(role_id)
        status, src_entity_id = utils.get_entity(self.options.url, self.auth,
                                                 self.options.src_entity)
        if not status:
            raise RuntimeError(src_entity_id)
        data['src_entity_id'] = src_entity_id
        status, dest_entity_id = utils.get_entity(self.options.url, self.auth,
                                                  self.options.dest_entity)
        if not status:
            raise RuntimeError(dest_entity_id)
        data['dest_entity_id'] = dest_entity_id
        r = requests.post(os.path.join(self.options.url, "role/share/add"),
                          auth=self.auth, json=data)
        if r.status_code != 200:
            raise RuntimeError('incorrect result from \'role/share/add\' service: %s' % r.text)
        res = 'ok'
        return res

    def delete(self):
        data = {}
        status, src_entity_id = utils.get_entity(self.options.url, self.auth,
                                                 self.options.src_entity)
        if not status:
            raise RuntimeError(src_entity_id)
        data['src_entity_id'] = src_entity_id
        status, dest_entity_id = utils.get_entity(self.options.url, self.auth,
                                                  self.options.dest_entity)
        if not status:
            raise RuntimeError(dest_entity_id)
        data['dest_entity_id'] = dest_entity_id
        for role in self.options.roles:
            status, role_id = utils.get_role(self.options.url, self.auth, role)
            if not status:
                raise RuntimeError(role_id)
            data['role_id'] = role_id
            r = requests.post(os.path.join(self.options.url, "role/share/remove"),
                              auth=self.auth, json=data)
            if r.status_code != 200:
                raise RuntimeError(
                    'incorrect result from \'role/share/remove\' service: %s' % r.text)
        res = 'ok'
        return res

    def execute_command(self):
        result = None
        if self.options.subcmd == "add":
            result = self.add()
        if self.options.subcmd == "delete":
            result = self.delete()
        return result



import spacy
import re
import argparse
import unicodedata
import re
from tqdm import tqdm

argparser = argparse.ArgumentParser()
argparser.add_argument('-f', '--file', required=True, type=str, help='Prefix of file to process')

args = argparser.parse_args()

f_fr = open(args.file+".fr")
f_en = open(args.file+".en")

print("#Loading entity models")
from spacy.lang.en import English
from spacy.lang.fr import French
nlp_fr = spacy.load("fr_core_news_sm")
nlp_en = spacy.load("en_core_web_sm")

f_mismatch = open(args.file+".fren-mismatch.txt", "w")
f_fr_keep = open(args.file+"-entkeep.fr", "w")
f_en_keep = open(args.file+"-entkeep.en", "w")

def norm_unicode(l):
    l = unicodedata.normalize('NFKD', l)
    l = u"".join([c for c in l if not unicodedata.combining(c)])
    l = l.lower().replace("ou", "u").replace("q","k")
    l = re.sub(r"([bcdfghjklmnpqrstvwxz])e(\b)", "\\1\\2", l.lower())
    return l

counter = 0
counter_with_person_entity = 0
mismatches = 0

count = 0
print("#Estimate file size")
for l_fr in f_fr:
    count = count + 1
f_fr.seek(0)

print("#Parse file %s,%s" % (args.file+".fr", args.file+".en"))
for (l_fr, l_en) in tqdm(zip(f_fr, f_en), total=count):
    doc_fr = nlp_fr(l_fr.strip())
    doc_en = nlp_en(l_en.strip())

    #print([e.text+"/"+e.label_ for e in doc_fr.ents]
    #	  , "--",
    #	  [e.text+"/"+e.label_ for e in doc_en.ents])

    ents_person_fr = [e.text for e in doc_fr.ents if e.label_=="PER"]
    ents_person_en = [e.text for e in doc_en.ents if e.label_=="PERSON"]

    if len(ents_person_en) > len(ents_person_fr):
        ok = True
        for e in ents_person_en:
            ok = ok and norm_unicode(l_fr).find(norm_unicode(e)) != -1
        if not ok:
            f_mismatch.write("\t".join([l_fr.strip(), ", ".join([a for a in ents_person_fr]),
                             			l_en.strip(), ", ".join([a for a in ents_person_en])])+"\n")
            mismatches += 1
        else:
            f_fr_keep.write(l_fr)
            f_en_keep.write(l_en)

    if len(ents_person_fr):
    	counter_with_person_entity += 1
    counter += 1
    if counter == 10000:
        break

print("#sentences", counter, "#withentity", counter_with_person_entity, "#mismatches", mismatches)



from prettytable import PrettyTable

import lib.utils as utils


def list_permission(args, auth):
    res = None
    is_json = args.display == "JSON"
    if args.subcmd == 'list':
        status_code, result = utils.permission_list(args.url, auth)
        if status_code != 200:
            raise RuntimeError(result)
        if not is_json:
            res = PrettyTable(["ID", "Name", "Description"])
            for r in result:
                if "entity_code" in r:
                    r["name"] = r["entity_code"] + ":" + r["name"]
                res.add_row([r["id"], r["name"], r["description"]])
        else:
            res = result
    return res



import os

import requests
from prettytable import PrettyTable

import lib.utils as utils


class Role:
    def __init__(self, options, auth):
        self.options = options
        self.auth = auth
        self.is_json = self.options.display == "JSON"

    def list(self):
        status_code, result = utils.role_list(self.options.url, self.auth)
        if status_code != 200:
            raise RuntimeError(result)
        if not self.is_json:
            res = PrettyTable(["ID", "Name", "Permissions", "Shared with"])
            res.align["Name"] = "l"
            res.align["Permissions"] = "l"
            res.align["Shared with"] = "l"
            for r in result:
                name = r["name"]
                if "entity_code" in r:
                    name = r["entity_code"] + ":" + name
                permissions = []
                for p in r["permissions"]:
                    if "entity_code" in p:
                        permissions.append(p["entity_code"] + ":" + p["permission"])
                    else:
                        permissions.append(p["permission"])

                shared_entities = []
                for entity in r["shared_entities"]:
                    shared_entities.append(entity["code"] + "(" + entity["name"] + ")")

                res.add_row([r["id"], name, " ".join(permissions), " ".join(shared_entities)])
        else:
            res = result
        return res

    def get_data(self):
        data = {
            'name': self.options.name
        }
        if self.options.permissions is None:
            if self.options.subcmd == 'add':
                raise ValueError("missing permissions")
        else:
            data['permissions'] = []
            for p in self.options.permissions:
                psplit = p.split(':')
                if len(psplit) > 2:
                    raise ValueError("invalid format for permission: %s" % p)
                status, permission_id = utils.get_permission(self.options.url,
                                                             self.auth, psplit[-1])
                if not status:
                    raise RuntimeError(permission_id)
                data["permissions"].append({'permission': permission_id})
                if len(psplit) == 2:
                    status, entity_id = utils.get_entity(self.options.url, self.auth, psplit[0])
                    if not status:
                        raise RuntimeError(permission_id)
                    data["permissions"][-1]["entity"] = entity_id
        return data

    def send_data(self, data):
        r = requests.post(os.path.join(self.options.url, "role/" + self.options.subcmd),
                          auth=self.auth, json=data)
        if r.status_code != 200:
            raise RuntimeError(
                'incorrect result from \'role/%s\' service: %s' % (self.options.subcmd, r.text))
        res = 'ok'
        return res

    def add(self):
        data = self.get_data()

        status, entity_id = utils.get_entity(self.options.url, self.auth, self.options.entity)
        if not status:
            raise RuntimeError(entity_id)
        data['entity_id'] = entity_id

        res = self.send_data(data)
        return res

    def modify(self):
        data = self.get_data()

        status, role_id = utils.get_role(self.options.url, self.auth, self.options.role)
        if not status:
            raise RuntimeError(role_id)
        data['role_id'] = role_id

        res = self.send_data(data)
        return res

    def delete(self):
        data = {}

        status, role_id = utils.get_role(self.options.url, self.auth, self.options.role)
        if not status:
            raise RuntimeError(role_id)
        data['role_id'] = role_id

        res = self.send_data(data)
        return res

    def execute_command(self):
        result = None
        if self.options.subcmd == "list":
            result = self.list()
        if self.options.subcmd == "add":
            result = self.add()
        if self.options.subcmd == "modify":
            result = self.modify()
        if self.options.subcmd == "delete":
            result = self.delete()
        return result



import os
import time

import configparser

if not os.path.exists('%s/.snw' % os.getenv('HOME')):
    os.mkdir('%s/.snw' % os.getenv('HOME'))


def _credential_file(url):
    norm_url = url.strip("/")
    p = norm_url.rfind("://")
    if p:
        norm_url = norm_url[p+3:]
    norm_url = norm_url.replace("/", "_")
    return '%s/.snw/credential-%s' % (os.getenv('HOME'), norm_url)


def get_credential(url, account=None):
    credential_file = _credential_file(url)
    config = configparser.ConfigParser()
    if os.path.exists(credential_file):
        config.read_file(open(credential_file))
    if not account:
        for section in config:
            if config[section].get("current"):
                account = section
                break
    if account in config:
        if float(config[account]["time"]) + float(config[account]["duration"]) >= time.time():
            return config[account]["token"], account
    return None, account


def set_credential(url, account, token, duration, time):
    credential_file = _credential_file(url)
    config = configparser.ConfigParser()
    if os.path.exists(credential_file):
        config.read_file(open(credential_file))
    for section in config:
        config.remove_option(section, "current")
    config[account] = {"token": token, "duration": duration, "time": time, "current": "True"}
    with open(credential_file, "w") as fh:
        config.write(fh)


def activate_credential(url, account):
    credential_file = _credential_file(url)
    config = configparser.ConfigParser()
    if os.path.exists(credential_file):
        config.read_file(open(credential_file))
    for section in config:
        config.remove_option(section, "current")
    config[account]["current"] = "True"
    with open(credential_file, "w") as fh:
        config.write(fh)


def remove_credential(url, account):
    credential_file = _credential_file(url)
    config = configparser.ConfigParser()
    if os.path.exists(credential_file):
        config.read_file(open(credential_file))
    if config.remove_section(account):
        with open(credential_file, "w") as fh:
            config.write(fh)


if __name__ == "__main__":
    set_credential("http://snw-api.net", "senellart@systran.fr", "1234")



import xml.etree.ElementTree as ET
import pcre

ns = {'srx20': 'http://www.lisa.org/srx20'}

class Segmenter(object):

    def __init__(self, srxfile, langcode, pure_srx=False, debug=False):
        tree = ET.parse(srxfile)
        root = tree.getroot()
        body = root.find('srx20:body', ns)
        self._lmap=[]
        rules=[]
        for child in body.find('srx20:maprules', ns).findall('srx20:languagemap', ns):
            if pcre.match(child.attrib["languagepattern"], langcode):
                self._lmap.append(child.attrib["languagerulename"])
        if debug:
            print("Matching languages:", ", ".join(self._lmap))
        for child in body.find('srx20:languagerules', ns).findall('srx20:languagerule', ns):
            if child.attrib["languagerulename"] in self._lmap:
                for rule in child.findall('srx20:rule', ns):
                    beforebreak=rule.find('srx20:beforebreak', ns)
                    if beforebreak is None or beforebreak.text is None:
                        beforebreak = ""
                    else:
                        beforebreak = beforebreak.text
                    afterbreak=rule.find('srx20:afterbreak', ns)
                    if afterbreak is None or afterbreak.text is None:
                        afterbreak = ""
                    else:
                        afterbreak = afterbreak.text
                    is_breaking=rule.attrib.get("break")=="yes"
                    regex = "("+beforebreak+")"+afterbreak
                    if pure_srx:
                        # Implement SRX regular expression definition
                        regex = regex.replace("\\s", "[\\t\\n\\f\\r\\p{Z}]")
                        regex = regex.replace("\\S", "[^\\t\\n\\f\\r\\p{Z}]")
                        regex = regex.replace("\\w", "[\\p{Ll}\\p{Lu}\\p{Lt}\\p{Lo}\\p{Nd}]")
                        regex = regex.replace("\\W", "[^\\p{Ll}\\p{Lu}\\p{Lt}\\p{Lo}\\p{Nd}]")
                    rules.append({"org": "  beforebreak: '%s'\n  afterbreak: '%s'" % (beforebreak, afterbreak),
                                  "re": pcre.compile(regex), "break": is_breaking})
        self._rules = rules
        self._debug = debug

    def get_langnames(self):
        return self._lmap

    def segment(self, text, trim=False):
        nobreakpos = {}
        breakpos = {}
        rid = 0
        for r in self._rules:
            rid += 1
            matches = r["re"].finditer(text)
            for m in matches:
                posbreak = m.start()+len(m.group(1))
                if self._debug:
                    print("Match [%s] in position %d-%d: \n%s"% (r["break"] and "break" or "nobreak", m.start(), posbreak, r["org"]))
                if posbreak not in breakpos and posbreak not in nobreakpos:
                    if r["break"]:
                        breakpos[posbreak]   = rid
                    else:   
                        nobreakpos[posbreak] = rid
        segments = []
        start_pos = 0
        for bp in sorted(breakpos.keys()):
            seq = text[start_pos:bp]
            if trim:
                seq = seq.strip()
            if len(seq):
                segments.append(seq)
            start_pos = bp
        seq = text[start_pos:]
        if trim:
            seq = seq.strip()
        if len(seq):
            segments.append(seq)
        return segments

